#include <atmel_start.h>
#include <stdio.h>

// Author : Gert van Biljon
// Email : gertvb@gmail.com
// Date : 2017/08/06
//
// What am I? : An Atmel Studio 7 example project for using the ADAFruit Feather M0 with Start.Atmel (ASF4)
//
// Why?
// 1. I LOVE the ARDUINO IDE as a teaching environment, but for teaching's sake it hides a lot of the functionality available on the ATSAMD21G18 chip,
//    and I want to access some of the hidden functionality, and with Atmel Studio all the hidden functionality is accessible.
// 2. I really battled to get started with configuring Atmel Studio for the Feather M0 (Particularly the USB Serial port, as I couldn't get the USB serial working 
//    as set up by Start.Atmel), and I figured that someone else might also battle.
//
// HOWTO
// =====
//
// 1. Ofsetting the start address of your program
// ==============================================
//
// You need to cater for the ADAFruit bootloader on your Feather M0 occupying the start of memory on your Feather M0 and Atmel Studio building your program for 
// the same address, by moving the start of your program to a higher address in memory than at 0, so that your program and the ADAFruit bootloader can coexist peacefully
// next to each other in memory.
//
// Solution Explorer -> Right Click on your project (Feather M0 in BOLD) -> Properties (right at the bottom) -> Toolchain -> Arm/Gnu linker -> Miscellaneous -> Linker Flags 
// modify from : -Tsamd21g18a_flash.ld to: -Wl,--section-start=.text=0x2000 -Tsamd21g18a_flash.ld
//
// Further info : https://reprapdad.wordpress.com/2016/08/19/adafruit-feather-m0-with-atmel-studio/
//
//
// 2. Install the ADAFruit USB Driver for Windows
// ==============================================
//
// When the Feather M0 is running the bootloader (Double pressing the reset button), the Feather M0 USB is started as an "ADAFruit Feather M0 device", and Windows needs to know
// what drivers to use to communicate with the Feather M0 when it is running the bootloader.
//
// Follow the instructions at : https://learn.adafruit.com/adafruit-arduino-ide-setup/windows-setup and install the Windows drivers
//
//
// 3. Installing the Bossac command line programmer
// ================================================
//
// Bossac is needed to write your compiled program to your Feather M0, it will download your program to the Feather M0 when the Feather M0 has 
// the bootloader program running on it. The bootloader is started on the Feather M0 by double-pressing the reset button on the Feather M0
//
// download : bossac-1.7.0-mingw32.tar.gz from : https://github.com/shumatech/BOSSA/releases
// extract bossac.exe to :  C:\Program Files (x86)\Bossac
//
// To create an entry for bossa in the Atmel Studio Tools menu, follow the instructions at : http://www.elecrom.com/program-arduino-due-atmel-studio/
// Note : do not add the line "mode %1......" to your batch file, it is for the dedicated serial port available on the Arduino Due
//
// Before you download the program to your Feather M0, you have to start the bootloader on your Feather M0 by double pressing the reset switch on the Feather M0!!
//
// More info on bossa : http://www.shumatech.com/web/products/bossa
//
// For those of us who hail from the command line era (and don't you dare laugh at our age!!) : bossac.exe -i -d --port=COM10 -U true -i -e -w -v LED_TOGGLE1.bin -R
//
//
// 4. Start.Atmel project creation
// ===============================
//
// I really battled to get the USB stuff going on my Feather M0 from an Atmel Studio Atmel Start project, my first breakthrough came when I built an AtSamd21 explained pro
// example project for the HID Mouse, and later for the CDC echo.
//
// It turns out that the culprit is in the clock setup, your USB needs to run at 48MHz and your CPU needs to run at 8MHz
//
// To see how this project's clocks are configured:
//
// Solution Explorer -> Right Click on your project (Feather M0 in BOLD) -> Reconfigure Atmel Start Project -> wait -> click on the clock icon on the left
//
//
// 5. ADC / A2D : Analog to digital converters
// ===========================================
//
// I am getting weird values from the ADC : 275 @ 0.0V and 3896 @ 3.316V
//
// I Suspect it has to do with the ADC Calibration Register, Bias_CAL, Linearity_cal values
//
//
// 6. Connections
// ==============
//
// Variable resistor (2kOhm) - Analogue Input
// ------------------------------------------
//
// 3.3v o--------o 
//                \
//                /
//                \
//                / <----^^^^^^^^(3kOhm)--------o AIN2 / PB08
//                \
//                /
//                \
//                / 
// GND  o--------o
//
//
// USB_COM_Port_Enable_Switch - GPIO Input
// ---------------------------------------
//
// 3.3V o--------o
//                 \
//                  o----------^^^^^^^^(3kOhm)--------o PA16
//
// GND  o--------o
//
//
// Frequency measurement - GPIO Output
// -----------------------------------
//
// PA15 o--------o
//               |
//               Multimeter
//               |
// GND  o--------o
//
//
// 
// Enjoy plenty!!
//
// Regards
//
// Gert

int main(void)
{
    char usb_serial_port_write_buffer[200];
    uint16_t batt_voltage_ACD_value;
    uint16_t variable_resistor_voltage_ADC_value;
    uint16_t temperature_ADC_value; 

    /* Initializes MCU, drivers and middleware */
    system_init();
    adc_sync_enable_channel(&ADC_0, 0);//Seems that the concept of channels is undefined on the samd21 ADC, thus using channel 0 throughout with the ADC

    /* Replace with your application code */
    while (1) {

        //If the user's usb on switch is on, then enable the usb com port, and write data on the USB com port
        if (gpio_get_pin_level(USB_COM_Port_Enable_Switch))
        {
            //This is an instance of a USB CDC (Communications Device Class) subset ACM (Abstract Control Model - Serial Port)
            //USB serial port
            usb_init();

            // wait for cdc acm to be enabled
            while ((!cdcdf_acm_is_enabled()) && (gpio_get_pin_level(USB_COM_Port_Enable_Switch)))
            {
                gpio_toggle_pin_level(Led);
                gpio_toggle_pin_level(Frequency_measuring_pin);             
                delay_ms(1);
            };

            //As long as the User keeps the usb switch on, keep on writing values to the com port
            while (gpio_get_pin_level(USB_COM_Port_Enable_Switch))
            {
                //Get the ADC values for the Battery, temperature and the variable resistor

                //There is something really weird with the ADC, still have a lot of figuring to do with the ADC
                //O.00V gives 292, and 3.314V gives 3896, with the values in between non linear ???????????
                //http://atmel-studio-doc.s3-website-us-east-1.amazonaws.com/webhelp/GUID-2A8AADED-413E-4021-AF0C-D99E61B8160D-en-US-1/index.html?GUID-23D99B59-B5EA-4187-A2C7-660DE4BEE77B
                adc_sync_set_inputs(&ADC_0, 0x07, 0x18, 0);//Read AIN7/A7/PA07 (0x07) referenced to Internal Ground (0x18)
                adc_sync_read_channel(&ADC_0, 0, (uint8_t*)&batt_voltage_ACD_value, 2);

                adc_sync_set_inputs(&ADC_0, 0x02, 0x18, 0);//from hpl_adc_config, read AIN2/A1/PB08 (0x02) referenced to Internal Ground (0x18)
                adc_sync_read_channel(&ADC_0, 0, (uint8_t*)&variable_resistor_voltage_ADC_value, 2);

                adc_sync_set_inputs(&ADC_0, 0x18, 0x18, 0);//Temperature (0x18) referenced to Internal Ground (0x18)
                adc_sync_read_channel(&ADC_0, 0, (uint8_t*)&temperature_ADC_value, 2);

                //You can read this text on a PC with a terminal program like putty
                snprintf(usb_serial_port_write_buffer, sizeof(usb_serial_port_write_buffer), "ADC Values : Variable Resistor : %04d, Battery : %04d, Temperature : %04d \n\r", variable_resistor_voltage_ADC_value, batt_voltage_ACD_value, temperature_ADC_value );
                cdcdf_acm_write((uint8_t *)usb_serial_port_write_buffer, strlen(usb_serial_port_write_buffer));

                gpio_toggle_pin_level(Led);
                gpio_toggle_pin_level(Frequency_measuring_pin);             
            }

            usbdc_deinit();
        }// if (gpio_get_pin_level(User_Switch))

        //gpio_set_pin_level(Led,gpio_get_pin_level(USB_On_Switch));
        delay_ms(500);
        gpio_toggle_pin_level(Led);
        gpio_toggle_pin_level(Frequency_measuring_pin);
    }// while (1) {
}
